function clu = clust_evol(clu,model,n_batch,opt_evo)
%CLUST_EVOL runs the fds evolving clustering algorithm.
%   CLU = CLUST_EVOL(CLU,MODEL,BATCH,OPT_EVO) add the parameter vector 
%   provided by MODEL arriving at BATCH in the cluster structure CLU,
%   options recorded in OPT_EVO are used
%
%   MODEL is generated with the function MODEL_CREATIION, while OPT_EVO is 
%   generated with PARAMETER_EVO
%
%   See also CLUST_TRAIN, INIT_CLUST,

%   Copyright 2013 Francesco Trovo' Politecnico di Milano
%   $Revision: 1.0$  $Date: 2013/04/04$

assert(opt_evo.eta_t == floor(opt_evo.eta_t));

test.coeff = model.res_fau;
n_test = length(model.eps_fau);
assert(n_test == 1);
% Extracting data
p = size(test.coeff,1);
    
mahal_nor = zeros(1,clu.n_clust_nor);

% Nominal clusters distance
time = n_batch;
new_data = test.coeff';
%Update global statistics
clu.samples = [clu.samples; new_data];
clu = update_glob_stat(clu,new_data);
clu.raw_data.Y = [clu.raw_data.Y model.Y{1}];
clu.raw_data.Z = [clu.raw_data.Z model.Z{1}];

for clus = 1:clu.n_clust_nor
    mahal_nor(clus) = mahalanobis(new_data,clu.med_nor{clus},clu.vari_nor{clus});
end

% Nominal data clustering
chosen_nor = find(mahal_nor == min(mahal_nor));
chosen_nor = chosen_nor(1);
n_nor = clu.n_points_nor(chosen_nor);
if opt_evo.verbose == 1
    fprintf('%s %f %s%f%s\n','Distance from nominal cluster:',mahal_nor(chosen_nor),...
        '(',(n_nor^2-1) * p / n_nor / (n_nor-p) * finv(1-opt_evo.alpha_s,p,n_nor-p),')');
end
% Checking spatial inclusion in the nominal cluster
if (mahal_nor(chosen_nor) <= (n_nor^2-1) * p / n_nor / (n_nor-p) * finv(1-opt_evo.alpha_s,p,n_nor-p))

    % Checking temporal inclusion in the nominal cluster
    if min(abs(clu.time_nor{chosen_nor} - time)) <= opt_evo.eta_t || ...
            (clu.n_added_point_nor(chosen_nor) ~= 0 && ...
            min(abs(clu.added_time_nor{chosen_nor} - time)) <= opt_evo.eta_t)

        clu.points_nor{chosen_nor} =  [clu.points_nor{chosen_nor}; new_data ];
        clu.n_points_nor(chosen_nor) = clu.n_points_nor(chosen_nor) + 1;
        clu.time_nor{chosen_nor} = [ clu.time_nor{chosen_nor} time ];
        if opt_evo.up_nom == 1
            % For static nominal states
            clu.med_nor{chosen_nor} = mean(clu.points_nor{chosen_nor});
            clu.vari_nor{chosen_nor} = update_cov(clu,chosen_nor,'n',opt_evo.up_cov);
        end
    else

        % Adding the point to the cluster but do not update it
        if clu.n_added_point_nor(chosen_nor) ~= 0
            clu.added_point_nor{chosen_nor} = [clu.added_point_nor{chosen_nor}; new_data];
            clu.n_added_point_nor(chosen_nor) = clu.n_added_point_nor(chosen_nor) + 1;
            clu.added_time_nor{chosen_nor} = [clu.added_time_nor{chosen_nor} time];
        else
            clu.added_point_nor{chosen_nor} = new_data;
            clu.n_added_point_nor(chosen_nor) = 1;
            clu.added_time_nor{chosen_nor} = time;
        end
    end
else

    if (clu.n_clust_fau >= 1)
        mahal_fau = zeros(1,clu.n_clust_fau);
        for clus = 1:clu.n_clust_fau
            mahal_fau(clus) = mahalanobis(new_data,clu.med_fau{clus},clu.vari_fau{clus});
        end
        chosen_fau = find(mahal_fau == min(mahal_fau));
        chosen_fau = chosen_fau(1);
        assert(chosen_fau <= clu.n_clust_fau);
        n_fau = clu.n_points_fau(chosen_fau);
    end

    % Checking spatial inclusion in the faulty clusters
    if (clu.n_clust_fau >= 1 && mahal_fau(chosen_fau) <= (n_fau^2-1) * p / n_fau / (n_fau-p)...
            * finv(1-opt_evo.alpha_s,p,n_fau-p))

        % Checking temporal inclusion in the faulty clusters
        if min(abs(clu.time_fau{chosen_fau} - time)) <= opt_evo.eta_t || ...
                ( clu.n_added_point_fau(chosen_fau) ~= 0 && ...
                min(clu.added_time_fau{chosen_fau} - time) <= opt_evo.eta_t)

            clu.points_fau{chosen_fau} =  [clu.points_fau{chosen_fau}; new_data ];
            clu.n_points_fau(chosen_fau) = clu.n_points_fau(chosen_fau) + 1;
            clu.time_fau{chosen_fau} = [ clu.time_fau{chosen_fau} time ];
            clu.med_fau{chosen_fau} = mean(clu.points_fau{chosen_fau});
            clu.vari_fau{chosen_fau} = update_cov(clu,chosen_fau,'f',opt_evo.up_cov);
            % Check if outliers are now in the modified cluster
            n_fau = clu.n_points_fau(chosen_fau);
            rem_index = 1:clu.n_outl;
            for out = 1:clu.n_outl
                data_out = clu.outliers(out,:);
                time_out = clu.outliers_time(out);
                mahal_out = mahalanobis(data_out, clu.med_fau{chosen_fau}, clu.vari_fau{chosen_fau});
                if ( mahal_out <= (n_fau^2-1) * p / n_fau / (n_fau-p) * finv(1-opt_evo.alpha_s,p,n_fau-p))

                    if min(abs(clu.time_fau{chosen_fau} - time_out)) <= opt_evo.eta_t || ...
                            ( clu.n_added_point_fau(chosen_fau) ~= 0 && ...
                            min(clu.added_time_fau{chosen_fau} - time_out) <= opt_evo.eta_t)

                        clu.points_fau{chosen_fau} =  [clu.points_fau{chosen_fau}; data_out];
                        clu.n_points_fau(chosen_fau) = clu.n_points_fau(chosen_fau) + 1;
                        clu.time_fau{chosen_fau} = [ clu.time_fau{chosen_fau} time_out ];
                        clu.med_fau{chosen_fau} = mean(clu.points_fau{chosen_fau});
                        clu.vari_fau{chosen_fau} = update_cov(clu,chosen_fau,'f',opt_evo.up_cov);
                    else
                        if clu.n_added_point_fau(chosen_fau) ~= 0
                            clu.added_point_fau{chosen_fau} = [ clu.added_point_fau{chosen_fau}; data_out];
                            clu.n_added_point_fau(chosen_fau) = clu.n_added_point_fau(chosen_fau) + 1;
                            clu.added_time_fau{chosen_fau} = [ clu.added_time_fau{chosen_fau} time_out ];
                        else
                            clu.added_point_fau{chosen_fau} = data_out;
                            clu.n_added_point_fau(chosen_fau) = 1;
                            clu.added_time_fau{chosen_fau} = time_out;
                        end
                    end
                    rem_index(out) = 0;
                end
            end

            rem_index = rem_index(rem_index ~= 0);
            if length(rem_index) ~= clu.n_outl
                % Updating the outlier set
                clu.outliers = clu.outliers(rem_index,:);
                clu.outliers_time = clu.outliers_time(rem_index);
                clu.n_outl = length(rem_index);
            end
            % Check if two fault clusters are overlapping
            clust_to_check = setdiff(1:clu.n_clust_fau,chosen_fau);
            merged = 0;
            for clust = clust_to_check
                if merged == 0
                    mu1 = clu.med_fau{clust};
                    mu2 = clu.med_fau{chosen_fau};
                    sigma1 = clu.vari_fau{clust};
                    sigma2 = clu.vari_fau{chosen_fau};
                    n1 = clu.n_points_fau(clust);
                    n2 = clu.n_points_fau(chosen_fau);
                end
                fault_distance1 = mahalanobis(mu1,mu2,sigma1);
                threshold1 = (n1 + 1) * (n2 - 1) * p / n2 / (n1 * n2 - n1 - p + 1) * ...
                        finv(1 - opt_evo.alpha_m / 2, p, n1 * n2 - n1 - p + 1);
                fault_distance2 = mahalanobis(mu1,mu2,sigma2);
                threshold2 = (n2 + 1) * (n1 - 1) * p / n1 / (n2 * n1 - n2 - p + 1) * ...
                        finv(1 - opt_evo.alpha_m / 2, p, n2 * n1 - n2 - p + 1);
                if opt_evo.verbose == 1
                    fprintf('%f %f %f %f\n',fault_distance1,threshold1,fault_distance2,threshold2);
                end
                if merged == 0 && fault_distance1 < threshold1 && fault_distance2 < threshold2
                    merged = 1;
                    clu.n_merge = clu.n_merge + 1;
                    %Merge the clusters
                    clust_ind = min([ clust chosen_fau]);
                    clust_to_del = max([ clust chosen_fau]);
                    clu.points_fau{clust_ind} = [clu.points_fau{clust}; clu.points_fau{chosen_fau}];
                    clu.time_fau{clust_ind} = [clu.time_fau{clust}, clu.time_fau{chosen_fau}];
                    clu.med_fau{clust_ind} = mean(clu.points_fau{clust_ind});
                    clu.vari_fau{clust_ind} = update_cov(clu,clust_ind,'f',opt_evo.up_cov);
                    clu.n_points_fau(clust_ind) = size(clu.points_fau{clust_ind},1);
                    clu.cre_time(clust_ind) = min([ clu.cre_time(clust) clu.cre_time(chosen_fau)]);
                    if clu.n_added_point_fau(clust_to_del) ~= 0
                        if clu.n_added_point_fau(clust_ind) ~= 0
                            clu.added_point_fau{clust_ind} = [clu.added_point_fau{clust}; clu.added_point_fau{chosen_fau}];
                            clu.n_added_point_fau(clust_ind) = clu.n_added_point_fau(clust) + clu.n_added_point_fau(chosen_fau);
                            clu.added_time_fau{clust_ind} = [clu.added_time_fau{clust} clu.added_time_fau{chosen_fau}];
                        else
                            clu.added_point_fau{clust_ind} = clu.added_point_fau{clust_to_del};
                            clu.n_added_point_fau(clust_ind) = clu.n_added_point_fau(clust_to_del);
                            clu.added_time_fau{clust_ind} = clu.added_time_fau{clust_to_del};
                        end
                    end
                    clu.points_fau(clust_to_del) = [];
                    clu.time_fau(clust_to_del) = [];
                    clu.med_fau(clust_to_del) = [];
                    clu.vari_fau(clust_to_del) = [];
                    clu.n_points_fau(clust_to_del) = [];
                    clu.cre_time(clust_to_del) = [];
                    if clu.n_added_point_fau(clust_to_del) ~= 0
                        clu.added_point_fau(clust_to_del) = [];
                        clu.added_time_fau(clust_to_del) = [];
                    end
                    clu.n_added_point_fau(clust_to_del) = [];
                    clu.n_clust_fau = clu.n_clust_fau - 1;
                end
            end
        else
            % Add the point to the other point of the cluster (when we add new constraints)
            if clu.n_added_point_fau(chosen_fau) ~= 0
                clu.added_point_fau{chosen_fau} = [clu.added_point_fau{chosen_fau}; new_data];
                clu.n_added_point_fau(chosen_fau) = clu.n_added_point_fau(chosen_fau) + 1;
                clu.added_time_fau{chosen_fau} = [ clu.added_time_fau{chosen_fau} time];
            else
                clu.added_point_fau{chosen_fau} = new_data;
                clu.n_added_point_fau(chosen_fau) = 1;
                clu.added_time_fau{chosen_fau} = time;
            end
        end

    else

        % If it is not near any cluster add it to outliers

        clu.n_outl = clu.n_outl + 1;
        clu.outliers(clu.n_outl,:) = new_data;
        clu.outliers_time(clu.n_outl) = time;
        % Check if there is a new cluster in formation
        if size(clu.outliers,1) > p
            %montecarlo_data = montecarlo_gmm(cl,opt_evo.n_montec_sample * (cl.n_clust_nor + cl.n_clust_fau));
            montecarlo_data = montecarlo_gmm(clu,opt_evo.n_montec_sample,opt_evo.alpha_s);
            data_emp = normalize(clu,clu.samples);
            data_sim = normalize(clu,montecarlo_data);
        if multi_kstest(data_emp,data_sim,opt_evo.alpha_c,opt_evo.verbose) == 1
%            if size(cl.outliers,1) > p && multi_kstest(cl.samples,montecarlo_data,opt.alpha_c) == 1
            if opt_evo.verbose == 1
                fprintf('%s\n','Kolmogorov-Smirnov test passed');
            end
            % Create a new fault cluster
            [~, indexes] = find_centers(clu,time,opt_evo);
            if length(indexes) > p
                if opt_evo.verbose == 1
                    fprintf('%s\n','Cluster creation');
                end
                chosen_ind = mcd_estimation(clu,indexes);
                clu.n_clust_fau = clu.n_clust_fau + 1;
                clu.clust_ret(clu.n_clust_fau) = time - min(clu.outliers_time(chosen_ind));
                clu.points_fau{clu.n_clust_fau} = [];
                for outl = chosen_ind'
                    clu.points_fau{clu.n_clust_fau} = [ clu.points_fau{clu.n_clust_fau}; clu.outliers(outl,:) ];
                end
                clu.med_fau{clu.n_clust_fau} = mean(clu.points_fau{clu.n_clust_fau});
                clu.n_points_fau(clu.n_clust_fau) = length(chosen_ind);
                clu.time_fau{clu.n_clust_fau} = clu.outliers_time(chosen_ind);
                clu.vari_fau{clu.n_clust_fau} = update_cov(clu,clu.n_clust_fau,'f',opt_evo.up_cov);
                clu.n_added_point_fau(clu.n_clust_fau) = 0;
                clu.cre_time(clu.n_clust_fau) = time;

                % Remove points from outliers
                not_chosen_ind = setdiff(1:size(clu.outliers,1),chosen_ind);
                clu.outliers = clu.outliers(not_chosen_ind,:);
                clu.n_outl = length(not_chosen_ind);
                clu.outliers_time = clu.outliers_time(not_chosen_ind);
            else
                if opt_evo.verbose == 1
                    fprintf('%s%i%s\n','Only ',length(indexes),' can be grouped in a cluster');
                end
            end
        end
        end
    end
end

if n_test > 0;
    clu.last_time_instant = time;
end
