function varargout = plot_clust_evo(clu,ax,opt_draw,varargin)
% Input
%   clu:        a cluster structure generated from init_clust()
%   axes:       a refeence to a ghraphuical axis to display graphics
%   opt_draw:   options for the evolving clustering algorithm projection
%               given by parameter_draw()
%   varargin:   if opt_draw.anim == 1 handler to the VideoWriter() object
%               recording the sequnce of frames of the algorithm
% Output
%   varargout:  if opt_draw.anim == 1 handler to the VideoWriter() updated 
%               object recording the sequnce of frames of the algorithm

if clu.n_points >= 3 && sum(clu.n_points_nor == 0) == 0
    p = size(clu.samples,2);
    if (p == 1 && strcmp(opt_draw.type,'nofig') == 0)
        fprintf('%s\n','Dimension too low to display graphics')
        opt_draw.type = 'nofig';
        opt_draw.anim = 0;
    elseif (p > 1 && p < 3 && strcmp(opt_draw.type,'3D') == 1)
        fprintf('%s\n','Dimension too low to display 3D graphics')
        opt_draw.type = '2D';
    end
    if strcmp(opt_draw.type,'2D')
        plot_clust_evo_2D(clu,ax,opt_draw);
    elseif strcmp(opt_draw.type,'3D')
        plot_clust_evo_3D(clu,ax,opt_draw);
    end
    drawnow;
end

if nargin == 4
    writerObj = varargin{1};
end
if ~strcmp(opt_draw.type,'nofig')
    if opt_draw.anim == 1
        axis tight;
        set(gca,'nextplot','replacechildren');
        set(gcf,'Renderer','zbuffer');
        frame = getframe(gcf);
        writeVideo(writerObj,frame);
    end
end

if nargout == 1
    varargout{1} = writerObj;
end

pause(opt_draw.wait);
