function [feature_sequence feature_data_index] = hmm_feature_extraction(input, output, opt_hmm, arx_order)

na = arx_order(1);
nb = arx_order(2);
nk = arx_order(3);

% Prepare data for model estimation
in = windowize(input, opt_hmm.data_window, opt_hmm.data_step); 
out = windowize(output, opt_hmm.data_window, opt_hmm.data_step);
feature_data_index = zeros(size(in,2),2);
for i = 1:size(in,2)
    feature_data_index(i,1) = 1 + (i-1) * opt_hmm.data_step;
    feature_data_index(i,2) = opt_hmm.data_window + (i-1) * opt_hmm.data_step;
end
n_feat = size(in,2);

% Model estimation (coefficients are feature for HMM)
feature_sequence = zeros(n_feat,na + nb + opt_hmm.const_input);
for index = 1:n_feat
    input = in(:,index)';
    output = out(:,index)';

    opt_est.eff_batch = length(input);
    opt_est.aord = na;
    opt_est.bord = nb;
    opt_est.kord = nk;
    opt_est.n_input = 1;
    opt_est.max_ord = max([na nb+nk]);
    opt_est.batch = opt_est.eff_batch - opt_est.max_ord;

    opt_est.mode = 'arx';
    opt_est.res_anal = 0;
    opt_est.const_input = opt_hmm.const_input;
    
    model = model_creation(input,output,opt_est);

    feature_sequence(index,:) = [model.res_fau(1:na)' model.res_fau(na+1:end)'];
end
