function [isol_delay isol_location] = hmm_isolation(model, aggr, detection_index, fault_time, detection_time, opt_hmm, fau_sensor, varargin)

if nargin > 7
    apost_threshold = varargin{1};
else
    apost_threshold = [];
end

model_detection = model(detection_index);
model(detection_index) = [];
det_input = model_detection.inputI;
det_output = model_detection.outputI;


%Partition the models
model_eplus = [];
model_eminus = [];
model_env_change = [];

for ii = 1:length(model)
    if model(ii).inputI == det_input || model(ii).outputI == det_input
        model_eplus = [model_eplus model(ii)];
    else
        if model(ii).inputI == det_output || model(ii).outputI == det_output
            model_eminus = [model_eminus model(ii)];
        else
            model_env_change = [model_env_change model(ii)];
        end
    end    
end

%Cognitive Isolation
epr_delay = [];
epl_delay = [];
emi_delay = [];

%E^p
if ~isempty(model_env_change)
    [epr_delay, isol_epr] = hmm_single_isolation(model_env_change, aggr, fault_time, detection_time, opt_hmm, apost_threshold);
end
%E^+
if ~isempty(model_eplus)
    [epl_delay, isol_epl] = hmm_single_isolation(model_eplus, aggr, fault_time, detection_time, opt_hmm, apost_threshold);
end
%E^-
if ~isempty(model_eminus)
    [emi_delay, isol_emi] = hmm_single_isolation(model_eminus, aggr, fault_time, detection_time, opt_hmm, apost_threshold);
end

if isempty(epr_delay) && isempty(epl_delay) && isempty(emi_delay)
    isol_location = NaN;
    isol_delay = -1;
else
    isol_delay = min([epr_delay epl_delay emi_delay]);
    if epr_delay - opt_hmm.env_wait <= isol_delay
        isol_location = -1;
    else
        if isol_delay == epl_delay
            isol_location = det_input;
        else
            isol_location = det_output;
        end 
    end        
end
