function model = train_hmm_fds(train_data, chosen_rel, ordersSel, opt_hmm)

% Parameters
% ------------------------------------------
% chosen_rel = couples of streams which should be considered
% ordersSel = orders for each relationship selected
%
% opt_hmm.states = [3 4 5 6];
% opt_hmm.data_window = 100;
% opt_hmm.data_step = 1;
% opt_hmm.par_window = 10;
% opt_hmm.par_step = 1;
% ------------------------------------------

%Check the rekationships
assert(size(chosen_rel,1) == size(ordersSel,1));

%Check on the input
if size(train_data,2) > size(train_data,1)
    train_data = train_data';
end

% Training the HMM model
n_rel = size(chosen_rel,1);
model = struct('hmm',[],'n_comp',[],'order',[],'inputI',[],'outputI',[]);

for jj = 1:n_rel
    % Feature extraction
    input = chosen_rel(jj,1);
    output = chosen_rel(jj,2);
    fprintf('Training relationship between the sensors (input -> output): %d -> %d\n', input,output);
    features = hmm_feature_extraction(train_data(:,input), train_data(:,output), opt_hmm, ordersSel(jj,:));
    [features, mu, sigma] = zscore(features);
    
    temp_hmm = train_single_hmm(features,opt_hmm);
    model(jj).mu = mu;
    model(jj).sigma = sigma;

    model(jj).hmm = temp_hmm;
    model(jj).n_comp = opt_hmm.dim_ens;
    model(jj).order = ordersSel(jj,:);
    model(jj).inputI = input;
    model(jj).outputI = output;
end
