function hmm = train_single_hmm(features,opt_hmm)
% function [model_name, model] ...
%     = train_single_hmm(data, training_area, validation_area, input_id,...
%     output_id, opt_hmm, arx_order)
% Input: 
%       features        : feature for the HMM training
%       opt_hmm         : option for the choice of the HMM
%
% Output:
%       hmm           : model containing the information of the HMM
%           - model_name
%           - modes
%           - states

states = opt_hmm.states;
gauss = opt_hmm.gauss;
n_feat = size(features,1);

n_train = floor(n_feat * opt_hmm.n_hmm_train/opt_hmm.n_hmm_valid);
n_valid = n_feat;
assert(n_valid > n_train);
train_features = features(1:n_train,:)';
valid_features = features((n_train+1):n_valid,:)';
valid_lls = zeros(length(states),length(gauss));
hmm = struct('prior',[],'transmat',[],'mu',[],'Sigma',[],'mixmat',[]);
valid_hmm = struct('prior',[],'transmat',[],'mu',[],'Sigma',[],'mixmat',[]);

switch opt_hmm.ens_type
    case 'etero'
        for kk = 1:opt_hmm.dim_ens
            for ii = 1:length(states)
                n_states = opt_hmm.states(ii);
                for jj = 1:length(gauss)
                    n_gauss = opt_hmm.gauss(jj);
                    [valid_lls(ii,jj) valid_hmm(ii,jj)] = train_hmm(train_features,valid_features,n_states,n_gauss);
                    if opt_hmm.n_repe > 1
                        for i = 2:opt_hmm.n_repe
                            [temp_lls temp_hmm] = train_hmm(train_features,valid_features,n_states,n_gauss);
                            if temp_lls > valid_lls(ii,jj)
                                valid_hmm(ii,jj) = temp_hmm;
                                valid_lls = temp_lls;
                            end
                        end
                    end
                end
            end
           %Best model choice
           [max_row max_col] = find(valid_lls ==(max(valid_lls(:))),1);
           hmm(kk) = valid_hmm(max_row,max_col);
        end
    case 'homo'
        for ii = 1:length(states)
            n_states = opt_hmm.states(ii);
            for jj = 1:length(gauss)
                n_gauss = opt_hmm.gauss(jj);
                [valid_lls(ii,jj) valid_hmm(ii,jj)] = train_hmm(train_features,valid_features,n_states,n_gauss);
                if opt_hmm.n_repe > 1
                    for i = 2:opt_hmm.n_repe
                        [temp_lls ~] = train_hmm(train_features,valid_features,n_states,n_gauss);
                        if temp_lls > valid_lls(ii,jj)
                            valid_lls = temp_lls;
                        end
                    end
                end
            end
        end
        [max_row max_col] = find(valid_lls ==(max(valid_lls(:))),1);
        n_states = states(max_row);
        n_gauss = gauss(max_col);
        lls = zeros(opt_hmm.dim_ens,1);
        for kk = 1:opt_hmm.dim_ens
            [lls(kk) hmm(kk)] = train_hmm(train_features,valid_features,n_states,n_gauss);            
        end
        [~, idx] = sort(lls);
        hmm = hmm(idx);
end
