function values = sel_rel(data,n_train,ordersSel,method,varargin)
% Input:
%
% Output
%   indexes: ordered indexes for the most relevant relationships (high == relevant)
%   values : for method = 'corr' crosscorrelation values for the
%   corresponding relationship, for method = 'test' pvalues

n_var = size(data,2);
dispos = dispositions(n_var);
n_rel = size(dispos,1);

switch method
    case 'test'
        batch = varargin{1};
        const_input = varargin{2};

        %Parameter vectors extraction
        max_aord = max(ordersSel(:,1));
        max_bord = max(ordersSel(:,2));
        max_kord = max(ordersSel(:,3));
        max_eff_batch = max([max_aord, max_bord + max_kord]) + batch;
        n_train_batch = floor(n_train / max_eff_batch );

        theta = cell(1,n_rel);
        for kk = 1:n_rel
            input = data(:,dispos(kk,1));
            output = data(:,dispos(kk,2));
            temp_batch = max_eff_batch - max([ordersSel(kk,1),ordersSel(kk,2)+ordersSel(kk,3)]);
            theta{kk} = parameter_extraction(output, input, temp_batch, const_input, ordersSel(kk,:));
        end
        [~, values] = test_exog(theta, n_train_batch, ordersSel, const_input, []);
    case 'test_single'
        const_input = varargin{1};

        %Parameter vectors extraction
        theta = cell(1,n_rel);
        cov_theta = cell(1,n_rel);
        for kk = 1:n_rel
            input = data(:,dispos(kk,1));
            output = data(:,dispos(kk,2));
            temp_batch = n_train - max([ordersSel(kk,1),ordersSel(kk,2)+ordersSel(kk,3)]);
            [theta{kk} cov_theta{kk} resid{kk}] = parameter_extraction(output, input, temp_batch, const_input, ordersSel(kk,:));
        end
        
        [~, values] = test_exog_total(theta, cov_theta, resid, n_train, ordersSel, const_input, []);
    case 'corr'
        train_data = data(1:n_train,:);
        [adj, cor] = corr_graph(train_data,[]);

        values = zeros(n_rel,1);
        for kk = 1:n_rel
            values(kk) = cor(dispos(kk,1),dispos(kk,2)) * adj(dispos(kk,1),dispos(kk,2));
        end
    case 'granger'
        opt_gra = varargin{1};
        [~, pvalue_gra] = granger(data',0.05,opt_gra);
        values = zeros(n_rel,1);
        for kk = 1:n_rel
            values(kk) = pvalue_gra(dispos(kk,2),dispos(kk,1));
        end
    case 'granger_prime'
        opt_gra = varargin{1};
       [~, pvalue_gra] = granger_prime(data',0.05,opt_gra);
        values = zeros(n_rel,1);
        for kk = 1:n_rel
            values(kk) = pvalue_gra(dispos(kk,2),dispos(kk,1));
        end
    case 'granger_pair'
        opt_gra = varargin{1};
        values = zeros(n_rel,1);
        for kk = 1:n_rel
            input = dispos(kk,1);
            output = dispos(kk,2);
            [~, pvalues] = granger(data(:,[input output])',0.05,opt_gra);
            values(kk) = pvalues(2,1);
        end
    case 'double_regress'
        n_input = n_var - 1;
        values = zeros(n_rel,1);
        for kk = 1:n_rel
            output = dispos(kk,1);
            compl_input = setdiff(1:n_var,output);
            reduc_input = setdiff(1:n_var,[output dispos(kk,2)]);
            data_xy = iddata(data(:,output),data(:,compl_input));
            data_y =  iddata(data(:,output),data(:,reduc_input));
            order_full = [ordersSel(kk,1) ordersSel(kk,2)*ones(1,n_input) ordersSel(kk,3)*ones(1,n_input)];
            order_redu = [ordersSel(kk,1) ordersSel(kk,2)*ones(1,n_input-1) ordersSel(kk,3)*ones(1,n_input-1)];
            model_full = arx(data_xy,order_full);
            model_red = arx(data_y,order_redu);
            RSS = model_full.Report.Fit.MSE;
            RSS_prime = model_red.Report.Fit.MSE;
            tot_par = sum([ordersSel(kk,1) ordersSel(kk,2)*ones(1,n_input)]);
            red_par = sum([ordersSel(kk,1) ordersSel(kk,2)*ones(1,n_input-1)]);
            F = (RSS_prime - RSS) / (RSS);
            values(kk) = 1 - fcdf(F / (tot_par - red_par) * (n_train - tot_par),tot_par - red_par,...
                n_train - tot_par);
        end
end
