function y = inject_fault(y,opt_fau)
% INJECT_FAULT inject a non structured fault in a signal
%   Injects a fault specificed in OPT_FAU in the signal Y
%   (monodimensional)
%
%   Input:
%       y: a generic monodimensional signal
%       opt_fau.time_profile: vector specifying the initial time for faults
%       opt_fau.type: vector specifying the type of the fault
%           'add' =     additive fault
%           'mult' =    multiplicative fault
%           'freeze' =  stuck-at fault
%           'degrad' =  degradation fault with (gaussian noise)
%       opt_fau.per: magnitude of the fault
%
%   Output:
%       y: faulty signal

maxi = max(y);
mini = min(y);
range = maxi - mini;

for i = 1:length(opt_fau.perc)
    init_fau = opt_fau.time_profile(1+(i-1)*2);
    end_fau = opt_fau.time_profile(i*2);
    if strcmp(opt_fau.type,'add')
        inte = range * opt_fau.perc;
        y(init_fau:end_fau) = y(init_fau:end_fau) + inte;
    elseif strcmp(opt_fau.type,'mult')
        media = mean(y);
        y(init_fau:end_fau) = (y(init_fau:end_fau) - media) * (1 + opt_fau.perc) + media;
    elseif strcmp(opt_fau.type,'freeze')
        y(init_fau:end_fau) = y(init_fau-1);
    elseif strcmp(opt_fau.type,'degrad')
        sigma = std(y) * opt_fau.perc;
        y(init_fau:end_fau) = y(init_fau:end_fau) + normrnd(0,sigma,end_fau-init_fau+1,1);
    end
end
