function [model opt_elm] = my_ELM(x,yf,opt_elm)
% Input:
%       x: input data
%       y: output nominal status data
%       yf: faulty data
%       opt_arx.batch: length of the batch for coefficients estimation
%       opt_arx.spost: movement of the window for estimation
%       opt_arx.not_fault: faulty-free
%       opt_arx.aord: autoregressive order for the arx
%       opt_arx.n_input: number of input
%       opt_arx.bord: order for the exogenous part of the arx
%       opt.detrend:
%                       0 No detrend
%                       1 Detrend for each batch
%
% Output:
%       model.res_nor: coefficients for the free fautl model
%       model.res_fau: coefficients fo the model with the fault
%       model.m_nory: estimated not faulty y mean
%       model.m_fauy: estimated faulty y mean
%       model.eps_nor: residual in not faulty condition
%       model.eps_fau: residual in faulty condition

% Initialization
opt_elm.max_order = max([opt_elm.aord opt_elm.bord]);
n = opt_elm.aord + opt_elm.bord * opt_elm.n_input;
assert(opt_elm.spost > 0 && opt_elm.spost <= opt_elm.batch); % Number of different samples from batch to batch (1 - batch)

opt_elm.disc_data = 2 * opt_elm.max_order; %Discarded samples (excluding unstable samples)
eff_batch = opt_elm.batch - opt_elm.max_order; % effective datasample available
if (opt_elm.spost == opt_elm.batch)
    opt_elm.n_samples = floor((opt_elm.hor - opt_elm.disc_data) / opt_elm.batch);
    opt_elm.n_train = max([0 floor((opt_elm.not_fault - opt_elm.disc_data) / opt_elm.batch)]);
else
    opt_elm.n_samples = floor((opt_elm.hor - opt_elm.disc_data - opt_elm.batch) / opt_elm.spost) + 1;
    opt_elm.n_train = max([0 floor((opt_elm.not_fault - opt_elm.disc_data - opt_elm.batch) / opt_elm.spost)]);
end

m_x = zeros(opt_elm.n_samples,opt_elm.n_input); % Estimated x mean

model.m_fauy = zeros(1,opt_elm.n_samples); % Estimated faulty y mean
model.res_fau = zeros(n,opt_elm.n_samples); % Estimated coefficients in faulty condition
model.eps_fau = zeros(1,opt_elm.n_samples); % Residual in not faulty condition
model.Z = cell(1,opt_elm.n_samples);
model.Y = cell(1,opt_elm.n_samples);

% Coefficients estimated with LS
for i = 0:(opt_elm.n_samples-1)
    % Data matrices initialization
    x_cons = zeros(opt_elm.bord*opt_elm.n_input,eff_batch);
    yf_cons = zeros(opt_elm.aord,eff_batch);
    if (opt_elm.spost == opt_elm.batch)
        init_index = opt_elm.disc_data + 1 + opt_elm.batch*i;
        final_index = opt_elm.disc_data + opt_elm.batch*(i+1);
    else
        init_index = opt_elm.disc_data + 1 + opt_elm.spost*i;
        final_index = opt_elm.disc_data + opt_elm.batch + opt_elm.spost*i;
    end
    % Mean calculation
    m_x(i+1,:) = mean( x(:,init_index:final_index),2);
    model.m_fauy(i+1) = mean(yf(init_index:final_index));

    % Data matrices construction
    yf_to_est = yf( (init_index + opt_elm.max_order) : final_index );
    for j = 0:(eff_batch-1)
        for k = 0:(opt_elm.n_input-1)
            x_cons(1+k*opt_elm.bord:(k+1)*opt_elm.bord,j+1) = ...
                x ( k+1, (init_index + opt_elm.max_order - opt_elm.bord + j) : (init_index + opt_elm.max_order - 1 + j) );
        end
        yf_cons(:,j+1) = yf( (init_index + opt_elm.max_order - opt_elm.aord + j) : (init_index + opt_elm.max_order - 1 + j) );
    end
    if (opt_elm.detrend == 1)
        yf_to_est = yf_to_est - model.m_fauy(i+1);
        yf_cons = yf_cons - model.m_fauy(i+1);
    end

    %LS with QR decomposition
    data_fau = [ yf_to_ext; yf_cons; x_cons ];
    
    [~, ~, model.esp_fau(i+1), ~, ]= elm_kernel(phitra_fau,data_fau,0,opt_elm.neurons,opt_elm.kernel,opt_elm.kernel_para);
    %R_fau = triu(qr([phitra_fau',yf_to_est']));
    %dimens = min(size(R_fau));
    %R_fau = R_fau(1:dimens,1:dimens);
    model.res_fau(:,i+1) = pinv(R_fau(1:dimens-1,1:dimens-1)) * R_fau(1:dimens-1,dimens);

    % Calcolo mse
    model.Z{i+1} = [yf_cons ; x_cons];
    model.Y{i+1} = yf_to_est;

    %model.eps_fau(i+1) = sum((model.res_fau(:,i+1)' * model.Z{i+1} - model.Y{i+1}).^2);

end
