function [model opt] = my_RN(x,yf,network,opt,opt_RN)
% Input:
%       x: input data
%       y: output nominal status data
%       yf: faulty data
%       opt_arx.batch: length of the batch for coefficients estimation
%       opt_arx.spost: movement of the window for estimation
%       opt_arx.not_fault: faulty-free
%       opt_arx.aord: autoregressive order for the arx
%       opt_arx.n_input: number of input
%       opt_arx.bord: order for the exogenous part of the arx
%       opt.detrend:
%                       0 No detrend
%                       1 Detrend for each batch
%
% Output:
%       model.res_nor: coefficients for the free fautl model
%       model.res_fau: coefficients fo the model with the fault
%       model.m_nory: estimated not faulty y mean
%       model.m_fauy: estimated faulty y mean
%       model.eps_nor: residual in not faulty condition
%       model.eps_fau: residual in faulty condition

% Initialization
opt.max_order = max([opt.aord opt.bord]);
n = opt_RN.n_units + opt_RN.direct * opt_RN.n_in;
assert(opt.spost > 0 && opt.spost <= opt.batch); % Number of different samples from batch to batch (1 - batch)

init_sample = 2 * opt.max_order; %Initial sample (excluding unstable samples)
eff_batch = opt.batch - opt.max_order; % effective datasample available
if (opt.spost == opt.batch)
    opt.n_samples = floor((opt.hor - init_sample) / opt.batch);
    opt.n_train = floor((opt.not_fault - init_sample) / opt.batch);
else
    opt.n_samples = floor((opt.hor - init_sample - opt.batch) / opt.spost) + 1;
    opt.n_train = floor((opt.not_fault - init_sample - opt.batch) / opt.spost);
end

m_x = zeros(opt.n_samples,opt.n_input); % Estimated x mean
%model.m_nory = zeros(1,opt.n_samples); % Estimated not faulty y mean
%model.res_nor = zeros(n,opt.n_samples); % Estimated coefficients in not faulty condition
model.m_fauy = zeros(1,opt.n_samples); % Estimated faulty y mean
model.res_fau = zeros(n,opt.n_samples); % Estimated coefficients in faulty condition
%model.eps_nor = zeros(1,opt.n_samples); % Residual in not faulty condition
model.eps_fau = zeros(1,opt.n_samples); % Residual in not faulty condition
model.Z = cell(1,opt.n_samples);
model.Y = cell(1,opt.n_samples);

%Network choice
for i = 1:10
    network = RN_init(opt_RN);
end


% Coefficients estimated with LS
for i = 0:(opt.n_samples-1)
    % Data matrices initialization
    x_cons = zeros(opt.bord*opt.n_input,eff_batch);
    %y_cons = zeros(opt.aord,eff_batch);
    yf_cons = zeros(opt.aord,eff_batch);
    if (opt.spost == opt.batch)
        init_index = init_sample + 1 + opt.batch*i;
        final_index = init_sample + opt.batch*(i+1);
    else
        init_index = init_sample + 1 + opt.spost*i;
        final_index = init_sample + opt.batch + opt.spost*i;
    end
    % Mean calculation
    m_x(i+1,:) = mean( x(:,init_index:final_index),2);
    %model.m_nory(i+1) = mean( y(init_index:final_index));
    model.m_fauy(i+1) = mean(yf(init_index:final_index));

    % Data matrices construction
    %y_to_est =   y( (init_index + opt.max_order) : final_index );
    yf_to_est = yf( (init_index + opt.max_order) : final_index );
    for j = 0:(eff_batch-1)
        for k = 0:(opt.n_input-1)
            x_cons(1+k*opt.bord:(k+1)*opt.bord,j+1) = ...
                x ( k+1, (init_index + opt.max_order - opt.bord + j) : (init_index + opt.max_order - 1 + j) );
        end
        %y_cons(:,j+1)  = y ( (init_index + opt.max_order - opt.aord + j) : (init_index + opt.max_order - 1 + j) );
        yf_cons(:,j+1) = yf( (init_index + opt.max_order - opt.aord + j) : (init_index + opt.max_order - 1 + j) );
    end
    if (opt.detrend == 1)
        %y_to_est = y_to_est - m_nory(i+1);
        yf_to_est = yf_to_est - m_fauy(i+1);
        %y_cons = y_cons - m_nory(i+1);
        yf_cons = yf_cons - m_fauy(i+1);
    end

    %%%% train the ESN
    network = RN_train(network,[x_cons; yf_cons],yf_to_est',opt_RN);
    model.res_fau(:,i+1) = network.W_out;

    % Calcolo mse
    model.Z{i+1} = [yf_cons ; x_cons];
    model.Y{i+1} = yf_to_est;
    
    [network y_pred] = RN_test(network,[x_cons; yf_cons],opt_RN);
    model.eps_fau(i+1) = NRMSE(y_pred,yf_to_est');

%     figure();
%     plot(y_pred);
%     hold on;
%     plot(yf_to_est,'r');
%     fprintf('%s%f\n','NRMSE: ',model.eps_fau(i+1));

end