function network = rn_train(network,u,y,opt_est)

% Method for training a multiple input single output RN (initialize by RN_init)

% INPUT
% network: previously generated by RN_init
% u: input (variable x samples)
% y: output (samples x 1)
% opt an option structure with:
%     tikh: tikhonov regularization for the inverse matrix
%     opt_RN.excl_points:
%     direct: includes (1) or escludes (0) the direct connections

%OUTPUT
% network with the following modified elements:
%     W_out: output weights (now trained)

n_train_samples = size(u,2);

% Adding direct cinnection if needed
if opt_est.direct == 0
    M = zeros(opt_est.n_node,n_train_samples-opt_est.forg_points);
else
    M = zeros(opt_est.n_node+opt_est.n_in,n_train_samples-opt_est.forg_points);
end
% Pseudoinverse matrix minimization (Least square method)
for i = 1:n_train_samples
    network.x = feval(network.f, network.W * network.x + network.W_in * u(:,i));
    if i > opt_est.forg_points
        if opt_est.direct == 0
            M(:,i-opt_est.forg_points) = network.x;
        else
            M(:,i-opt_est.forg_points) = [network.x; u(:,i)];
        end
    end
end
r = feval(network.f_out_inv,y((1+opt_est.forg_points):n_train_samples));

network.W_out = (pinv(M*M' + opt_est.tikh * eye(size(M,1))) * M * r)';
